from __future__ import with_statement
from os.path import basename

from django.template import loader, Context
from django.core.mail import EmailMultiAlternatives

from email_extras.settings import USE_GNUPG, GNUPG_HOME, ALWAYS_TRUST


if USE_GNUPG:
    from gnupg import GPG


def addresses_for_key(gpg, key):
    """
    Takes a key and extracts the email addresses for it.
    """
    fingerprint = key["fingerprint"]
    addresses = []
    for key in gpg.list_keys():
        if key["fingerprint"] == fingerprint:
            addresses.extend([address.split("<")[-1].strip(">")
                              for address in key["uids"] if address])
    return addresses


def send_mail(subject, body_text, addr_from, addr_to, fail_silently=False,
              attachments=None, body_html=None):
    """
    Sends a multipart email containing text and html versions which
    are encrypted for each recipient that has a valid gpg key
    installed.
    """

    # Allow for a single address to be passed in.
    if not hasattr(addr_to, "__iter__"):
        addr_to = [addr_to]

    # Obtain a list of the recipients that have gpg keys installed.
    key_addresses = {}
    if USE_GNUPG:
        from email_extras.models import Address
        for address in Address.objects.filter(address__in=addr_to):
            key_addresses[address.address] = address.use_asc
        # Create the gpg object.
        if key_addresses:
            gpg = GPG(gnupghome=GNUPG_HOME)

    # Encrypts body if recipient has a gpg key installed.
    def encrypt_if_key(body, addr):
        if addr in key_addresses:
            return unicode(gpg.encrypt(body, addr, always_trust=ALWAYS_TRUST))
        return body

    # Load attachments and create name/data tuples.
    attachments_parts = []
    if attachments is not None:
        for attachment in attachments:
            # Attachments can be pairs of name/data, or filesystem paths.
            if not hasattr(attachment, "__iter__"):
                with open(attachment, "rb") as f:
                    attachments_parts.append((basename(attachment), f.read()))
            else:
                attachments_parts.append(attachment)

    # Send emails.
    for addr in addr_to:
        msg = EmailMultiAlternatives(subject, encrypt_if_key(body_text, addr),
                                     addr_from, [addr])
        if body_html is not None:
            body_html = encrypt_if_key(body_html, addr)
            msg.attach_alternative(body_html, "text/html")
        for parts in attachments_parts:
            name = parts[0]
            if key_addresses.get(addr):
                name += ".asc"
            msg.attach(name, encrypt_if_key(parts[1], addr))
        msg.send(fail_silently=fail_silently)


def send_mail_template(subject, template, addr_from, addr_to,
                       fail_silently=False, attachments=None, context=None):
    """
    Send email rendering text and html versions for the specified
    template name using the context dictionary passed in.
    """

    if context is None:
        context = {}

    # Loads a template passing in vars as context.
    def render(ext):
        name = "email_extras/%s.%s" % (template, ext)
        return loader.get_template(name).render(Context(context))

    send_mail(subject, render("txt"), addr_from, addr_to,
              fail_silently=fail_silently, attachments=attachments,
              body_html=render("html"))




from django import forms
from django.utils.translation import ugettext_lazy as _

from email_extras.settings import USE_GNUPG, GNUPG_HOME

if USE_GNUPG:
    from gnupg import GPG


class KeyForm(forms.ModelForm):

    def clean_key(self):
        """
        Validate the key contains an email address.
        """
        key = self.cleaned_data["key"]
        gpg = GPG(gnupghome=GNUPG_HOME)
        result = gpg.import_keys(key)
        if result.count == 0:
            raise forms.ValidationError(_("Invalid Key"))
        return key




from email_extras.settings import USE_GNUPG


if USE_GNUPG:
    from django.contrib import admin
    from email_extras.models import Key, Address
    from email_extras.forms import KeyForm

    class KeyAdmin(admin.ModelAdmin):
        form = KeyForm

    admin.site.register(Key, KeyAdmin)
    admin.site.register(Address)




from django.db import models
from django.utils.translation import ugettext_lazy as _

from email_extras.settings import USE_GNUPG, GNUPG_HOME
from email_extras.utils import addresses_for_key


if USE_GNUPG:
    from gnupg import GPG

    class Key(models.Model):
        """
        Accepts a key and imports it via admin's save_model which
        omits saving.
        """

        class Meta:
            verbose_name = _("Key")
            verbose_name_plural = _("Keys")

        key = models.TextField()
        addresses = models.TextField(editable=False)
        use_asc = models.BooleanField(default=False, help_text=_("If True, "
            "an '.asc' extension will be added to email attachments sent "
            "to the address for this key."))

        def __unicode__(self):
            return self.addresses

        def save(self, *args, **kwargs):
            super(Key, self).save(*args, **kwargs)
            gpg = GPG(gnupghome=GNUPG_HOME)
            result = gpg.import_keys(self.key)
            addresses = []
            for key in result.results:
                addresses.extend(addresses_for_key(gpg, key))
            self.addresses = ",".join(addresses)
            for address in addresses:
                address, _ = Address.objects.get_or_create(address=address)
                address.use_asc = self.use_asc
                address.save()

    class Address(models.Model):
        """
        Stores the address for a successfully imported key and allows
        deletion.
        """

        class Meta:
            verbose_name = _("Address")
            verbose_name_plural = _("Addresses")

        address = models.CharField(max_length=200)
        use_asc = models.BooleanField(default=False, editable=False)

        def __unicode__(self):
            return self.address

        def delete(self):
            """
            Remove any keys for this address.
            """
            from email_extras.utils import addresses_for_key
            gpg = GPG(gnupghome=GNUPG_HOME)
            for key in gpg.list_keys():
                if self.address in addresses_for_key(gpg, key):
                    gpg.delete_keys(key["fingerprint"], True)
                    gpg.delete_keys(key["fingerprint"])
            super(Address, self).delete()




from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


GNUPG_HOME = getattr(settings, "EMAIL_EXTRAS_GNUPG_HOME", None)
USE_GNUPG = getattr(settings, "EMAIL_EXTRAS_USE_GNUPG", GNUPG_HOME is not None)
ALWAYS_TRUST = getattr(settings, "EMAIL_EXTRAS_ALWAYS_TRUST_KEYS", False)


if USE_GNUPG:
    try:
        import gnupg
    except ImportError:
        raise ImproperlyConfigured("Could not import gnupg")




from setuptools import setup


setup(
    name = "django-email-extras",
    version = __import__("email_extras").__version__,
    author = "Stephen McDonald",
    author_email = "steve@jupo.org",
    description = "A Django reusable app providing the ability to "
                  "send PGP encrypted and multipart emails using "
                  "the Django templating system.",
    long_description = open("README.rst").read(),
    url = "https://github.com/stephenmcd/django-email-extras",
    packages = ["email_extras",],
    zip_safe = False,
    include_package_data = True,
    install_requires=["python-gnupg", "sphinx-me",],
    classifiers = [
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Framework :: Django",
        "Topic :: Communications :: Email",
        "Topic :: Security :: Cryptography",
    ]
)
